"""Onionr - Private P2P Communication.

Identify a data directory for Onionr
"""
import os
import platform
import uuid
"""
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


def identify_home() -> str:
    """Return path of home directory.

    Take env into account with sane OS defaults
    """
    path = os.environ.get('ONIONR_HOME', None)

    if path is None:
        system = platform.system()
        if system == 'Linux':
            path = os.environ.get(
                'XDG_DATA_HOME',
                os.path.expanduser('~') + '/.local/share/onionr/')
        elif system == 'Darwin':
            path = os.path.expanduser('~' +
                                      '/Library/Application Support/onionr/')
        else:
            path = 'data/'
    else:
        path = os.path.abspath(path)
    if not path.endswith('/'):
        path += '/'

    return path
